

/* C version of program to compute g(r) from stored configurations.
  It reads system parameters from the .chk file and
  reads configurations from the .crd file.  It calls accumgr after each
  configuration has been read.  At the end it calls write_gofr to
  normalize g(r) and to write it to a file.
*/
/* For this assignment you will need to fill in the accumgr and write_gofr
   functions */
   
/* Compile with cc -o cgofr cgofr.c -lm 
    (on HP, use c89 rather than cc)  */

 
#include <stdio.h>
#include <math.h>

#define minval(a,b) (a<b) ? (a) : (b)

struct point{
   double x[3];
};
void accumgr(struct point atom[],int ndim,int natoms,double ell[], double dr,
             int gofr[],int grsize);
void write_gofr(int ndim, int natoms, double ell[], double dr, int gofr[], 
                int nconfig, char *grid, int grsize);

int main()
{
   char runid[9],crdid[11],chkid[11],tdat[33],junk[256],grid[10];
   FILE *chk_file,*crd_file;
   int nread,ifply,kx,itype,nskip,grsize,nconfig;
   int nsets,ndone,ifmd,nblock;
   int ndim,natoms,ntypes,nunits;
   double beta,ro,temp,ell[3];
   double x1,x2,dr,cutr;
   float y1,y2,y3,tmp;
   struct point *atom;
   int *gofr;
   int i;

   printf(" gofr: input run id (lt 8 characters)\n");
   scanf("%s",runid);
   sprintf(chkid,"%s.chk",runid);
   sprintf(crdid,"%s.crd",runid);
   sprintf(grid,"%s.gr",runid);
   printf("Trying to open file %s\n",chkid);
   if ((chk_file = fopen(chkid,"r"))==NULL) {
      perror("Unable to open");
      exit(1);
   }


   fseek(chk_file,4,SEEK_CUR);   
   nread = fread(runid,1,8,chk_file);
   nread = fread(tdat,1,26,chk_file);
   tdat[26]=0;
   printf("runid: %8s  info: %26s \n",runid,tdat);
   nread = fread(&ifply,4,1,chk_file);
   nread = fread(&kx,4,1,chk_file);
   printf("ifply: %d  kx: %d \n",ifply,kx);

   fseek(chk_file,kx*8,SEEK_CUR); /* skip the positions */
   fseek(chk_file,8,SEEK_CUR);    /*skip the blank */

   nread = fread(&nsets,4,1,chk_file);
   nread = fread(&ndone,4,1,chk_file);
   nread = fread(&ifmd,4,1,chk_file);
   nread = fread(&nblock,4,1,chk_file);
   printf("nsets: %d ndone: %d ifmd %d nblock:%d\n",nsets,ndone,ifmd,nblock);
   fseek(chk_file,8,SEEK_CUR);    /*skip the blank */

/*   nread = fread(tdat,1,32,chk_file);
   for (i=0;i<32;i++){
     printf("i: %d val: %d %#x\n",i,tdat[i],tdat[i]);
   }*/

   ifmd=1;
   if (ifmd!=0) fseek(chk_file,8*kx,SEEK_CUR); /* skip velocities */
   
  /*   for (i=0;i<kx/2;i++){
       nread = fread(&x1,8,1,chk_file);
       nread = fread(&x2,8,1,chk_file);
       printf("i:%d x1:%g x2:%g \n",i,x1,x2);
    }*/
  /* nread = fread(tdat,1,8,chk_file);
   for (i=0;i<8;i++){
     printf("i: %d val: %d %#x\n",i,tdat[i],tdat[i]);
   }*/
   fseek(chk_file,8,SEEK_CUR);  /* skip the blank */

   nread = fread(&ndim,4,1,chk_file);
   nread = fread(&natoms,4,1,chk_file);
   nread = fread(&ntypes,4,1,chk_file);
   nread = fread(&beta,8,1,chk_file);
   nread = fread(&ro,8,1,chk_file);
   nread = fread(&temp,8,1,chk_file);
   nread = fread(&nunits,4,1,chk_file);
   
   printf("ndim: %d natoms: %d ntypes: %d \n",
                                      ndim,natoms,ntypes);
   printf("ro: %g temp: %g \n",ro,temp);
   fseek(chk_file,nunits*8,SEEK_CUR);  /* skip the uname */
   fseek(chk_file,ntypes*36,SEEK_CUR);  /* skip the stuff for each type */
   fseek(chk_file,4,SEEK_CUR);  /* skip nt2 */

   nread = fread(ell,8,ndim,chk_file);
   if (ndim==2) printf("ell1 %g ell2 %g\n",ell[0],ell[1]);
   if (ndim==3) printf("ell1 %g ell2 %g ell3 %g\n",ell[0],ell[1],ell[2]);

   /* then we would read stuff about the interactions, but it's not
      needed so we won't */


  /* Now we need to open the .crd file and read it */
 
   printf("Number of configurations to skip in beginning\n");
   scanf("%d",&nskip);
   printf("g(r) table spacing\n");
   scanf("%g",&tmp);
   dr = (double)tmp;

   cutr = 0.5*ell[0];
   if (ndim==2) cutr = minval(cutr,0.5*ell[1]);
   if (ndim==3) cutr = minval(cutr,0.5*ell[2]);
   if (dr<0.0) dr = 0.01*cutr;

   grsize = cutr/dr + 1;
   printf("cutr %g grsize %d\n",cutr,grsize);
   if (grsize > 20000) {
      printf("grsize too big: %d > 20000.  Choose a larger dr\n",grsize);
      exit(0);
   }
   gofr = (int *)malloc(grsize*sizeof(int));


   atom = (struct point *)malloc(natoms*sizeof(struct point));


   printf("Trying to open file %s\n",crdid);
   if ((crd_file = fopen(crdid,"r"))==NULL) {
      perror("Unable to open");
   }

 /* throw away the first two lines */
    fgets(junk,256,crd_file);
    fgets(junk,256,crd_file);

  ndone=0; nconfig=0;
  while( !feof(crd_file) ) {
     for(i=0;i<natoms;i++){
      if (ndim==2) {
         fscanf(crd_file,"%d %g %g",&itype,&y1,&y2);
         if (feof(crd_file)) goto outahere;
         atom[i].x[0] = (double)y1;
         atom[i].x[1] = (double)y2;
      } else if (ndim==3) {
         fscanf(crd_file,"%d %g %g %g",&itype,&y1,&y2,&y3);
         if (feof(crd_file)) goto outahere;
         atom[i].x[0] = (double)y1;
         atom[i].x[1] = (double)y2;
         atom[i].x[2] = (double)y3;
      }
     }
     ndone++;

    /* call the g(r) histogram accumulation routine here */
     if (ndone>nskip) {
         nconfig++;
         accumgr(atom,ndim,natoms,ell,dr,gofr,grsize);
     }
  }
   outahere:

  /* normalize  g(r) and write it out */

   write_gofr(ndim,natoms,ell,dr,gofr,nconfig,grid,grsize);
         
    

}
/* paramters 
   atom - contains the atomic configurations. Note that the point structure
    is defined at the top of this file. atom[i].x[j] will access the
    jth  dimension of atom i.
   ndim - number of dimensions
   natoms - number of atoms
   ell - size of box
   dr - size of each bin
   gofr - histogram for g(r)
   grsize - length of gofr
*/
  

void accumgr(struct point atom[],int ndim,int natoms,double ell[], double dr,
             int gofr[],int grsize)
{

/* You will need to compute the distances using the positions in atom
   (don't forget about periodic boundary conditions), bin them, and
    accumulate the results in gofr
*/


}
/* parameters
   ndim - number of dimensions
   natoms - number of atoms
   ell - box size
   dr - bin size
   gofr - histogram for g(r)
   nconfig - number of configurations used.  This will be the number of
             configurations in the .crd file minus the number skipped
             at the beginning.
   grid - the name of the g(r) file = runid.gr
   grsize - the length of gofr
*/

void write_gofr(int ndim, int natoms, double ell[], double dr, int gofr[], 
                int nconfig, char *grid, int grsize)
{

    double pi = 3.14159265358979;
    double norm;
    int i;
    FILE *grfile;
    if ((grfile = fopen(grid,"w"))==NULL) {
      perror("Unable to open");
    }


    for (i=0;i<grsize;i++){

/* you will need to compute the normalization here */

      fprintf(grfile,"%g %g %d\n",dr*(i+.5), norm*gofr[i],gofr[i]);

    }

    close(grfile);

    
}
 
